/*
 * jEdit editor settings:
 * :mode=java:tabSize=4:indentSize=4:noTabs=true:maxLineLen=0:
 *
 * JIndexPlugin.java - JIndex plugin
 * Copyright (C) 1999 Dirk Moebius
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


import java.util.Vector;
import java.util.Enumeration;
import javax.swing.*;

import org.gjt.sp.jedit.*;
import org.gjt.sp.jedit.gui.DockableWindow;
import org.gjt.sp.jedit.gui.DockableWindowManager;
import org.gjt.sp.jedit.gui.OptionsDialog;
import org.gjt.sp.jedit.msg.CreateDockableWindow;
import org.gjt.sp.jedit.msg.ShowTokenInfo;
import org.gjt.sp.util.Log;

import jindex.JIndexOptionPane;
import jindex.JIndexHoldable;
import jindex.MultipleEntriesDialog;


/**
 * A plugin for accessing JavaDoc generated API documentation via
 * keywords in the jEdit textarea.
 *
 * @author <A HREF="mailto:dmoebius@gmx.net">Dirk Moebius</A>
 */
public class JIndexPlugin extends EBPlugin {

    public final static String NAME = "plugin.JIndexPlugin";
    private static JIndexPlugin instance;
    private MultipleEntriesDialog dlg = null;


    public void start() {
        instance = this;
        EditBus.addToNamedList(DockableWindow.DOCKABLE_WINDOW_LIST, NAME);
    }


    public void createMenuItems(Vector menuItems) {
        menuItems.addElement(GUIUtilities.loadMenu("jindex-menu"));
    }


    public void createOptionPanes(OptionsDialog optionsDialog) {
        optionsDialog.addOptionPane(new JIndexOptionPane());
    }


    /**
     * JIndex listens to ShowTokenInfo messages with language 'java'.
     * @param message the message is vetoed if it is of type ShowTokenInfo
     *                and the language is 'java' (in lowercase).
     * @see org.gjt.sp.jedit.msg.ShowTokenInfo
     */
    public void handleMessage(EBMessage message) {
        if (message instanceof ShowTokenInfo) {
            ShowTokenInfo msg = (ShowTokenInfo) message;
            if (!"java".equals(msg.getLanguage().toLowerCase())) {
                // this message is not for java
                return;
            }
            if (msg.isVetoed()) {
                // someone else has handled this message
                return;
            }
            // accept the message:
            msg.veto();
            // search for JIndexHoldable instance in current view:
            DockableWindowManager mgr = msg.getView().getDockableWindowManager();
            if (mgr != null && mgr.isDockableWindowVisible(NAME)) {
                // a docked JIndexHoldable is visible in the current view:
                mgr.showDockableWindow(NAME);
                DockableWindow win = mgr.getDockableWindow(NAME);
                JIndexHoldable h = (JIndexHoldable) win.getComponent();
                h.search(msg.getToken());
            } else {
                // use the MultipleEntriesDialog
                if (dlg == null) {
                    dlg = new MultipleEntriesDialog(msg.getView(),
                                                    msg.getToken());
                }
                dlg.search(msg.getToken());
            }
        }
        else if (message instanceof CreateDockableWindow) {
            // jEdit requests an instance of a dockable JIndex window:
            CreateDockableWindow msg = (CreateDockableWindow) message;
            String name = msg.getDockableWindowName();
            if (name.equals(NAME)) {
                JIndexHoldable jh = new JIndexHoldable();
                msg.setDockableWindow(jh);
            }
        }
    }


    public final static JIndexPlugin getInstance() {
        return instance;
    }
}
