/*
 * jEdit editor settings:
 * :mode=java:tabSize=4:indentSize=4:noTabs=true:maxLineLen=0:
 *
 * JIndexCreator.java
 * Copyright (C) 1999 2000 Dirk Moebius
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jindex;

import java.awt.Dialog;
import java.io.*;
import org.gjt.sp.jedit.*;


public class JIndexCreator extends Thread implements EBComponent {

    private LibEntry[] libs = null;
    private String fname = null;
    private Dialog parent = null;
    private boolean closeParentWindowIfReady = false;


    public JIndexCreator(
            LibEntry[] libs,
            String indexfilename,
            Dialog parent,
            boolean closeParentWindowIfReady)
    {
        this.libs = libs;
        this.fname = indexfilename;
        this.parent = parent;
        this.closeParentWindowIfReady = closeParentWindowIfReady;
    }


    public void run() {
        ProgressWindow progwin = new ProgressWindow(parent, "Creating index");
        progwin.setString("Creating index. Please wait...");

        yield();
        JIndex index = new JIndex(libs, progwin);
        yield();
        progwin.setString("Writing index out to: " + fname + " ...");

        try {
            // create output stream:
            FileWriter fw = new FileWriter(fname);
            BufferedWriter bw = new BufferedWriter(fw);
            // write index out:
            index.writeXML(bw);
            bw.flush();
            bw.close();
        }
        catch (IOException e) {
            progwin.dispose();
            GUIUtilities.error(null, "jindex.error.write",
                new Object[] { fname, e } );
            return;
        }

        progwin.advance();

        // notify all listeners that the index has changed.
        EditBus.send(new JIndexChange(this, index));

        index = null;
        progwin.dispose();
        System.gc();

        if (closeParentWindowIfReady)
            parent.setVisible(false);
    }


    public void handleMessage(EBMessage message) { }

}
