/*
 * jEdit editor settings:
 * :mode=java:tabSize=4:indentSize=4:noTabs=true:maxLineLen=0:
 *
 * ProgressWindow.java - a window without handles with progress bar and status text
 * Copyright (C) 1999 Dirk Moebius
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jindex;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;


// Note: ProgressWindow may not be a modal dialog, or otherwise
// it would block the execution of the program until hide() or
// dispose().

public class ProgressWindow extends JDialog {

    Window parent = null;
    JLabel caption = null;
    JProgressBar pb = null;
    Cursor saveCursor = null;


    public ProgressWindow(Frame parent, String title) {
        super(parent, false);
        init(parent, title);
    }


    public ProgressWindow(Dialog parent, String title) {
        init(parent, title);
    }


    private void init(Window parent, String title) {
        this.parent = parent;
        setTitle(title);
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                Toolkit.getDefaultToolkit().beep();
            }
        });

        caption = new JLabel("Please wait...");
        caption.setBorder(new EmptyBorder(30, 30, 20, 30));
        caption.setPreferredSize(new Dimension(500, caption.getPreferredSize().height));

        pb = new JProgressBar();
        pb.setValue(0);
        pb.setStringPainted(true);
        pb.setString("0%");
        pb.setBorder(new EmptyBorder(0, 30, 30, 30));

        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(caption, BorderLayout.CENTER);
        getContentPane().add(pb, BorderLayout.SOUTH);

        saveCursor = parent.getCursor();
        Cursor newCursor = Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR);
        parent.setCursor(newCursor);
        setCursor(newCursor);

        pack();
        setLocationRelativeTo(parent);
        setVisible(true);
    }


    public void setMaximum(int max) {
        if (max > 0)
            pb.setMaximum(max);
    }


    public void setString(final String s) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                caption.setText(s);
            }
        });
    }


    public void advance() {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                int newVal = pb.getValue() + 1;
                int maxVal = pb.getMaximum();
                int percentComplete = (int) ((((double)newVal) / ((double)maxVal)) * 100.0);
                pb.setValue(newVal);
                pb.setString("" + percentComplete + "%");
            }
        });
    }


    public void setVisible(boolean visible) {
        if (!visible) {
            setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            parent.setCursor(saveCursor);
        }
        super.setVisible(visible);
    }


    public void dispose() {
        setVisible(false);
        super.dispose();
    }
}
